/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/MainW.h>
#include <Xm/LabelG.h>
#include <X11/xpm.h>

#include <stdio.h>
#include "../print.h"

typedef struct {
    short r;
    short g;
    short b;
    short w;
    short color;
    char file[10];
    GC brush;
} Symbol;


int debug = 1;

#define SCREEN 0
#define POSTSCRIPT 1
#define COLORPS 2

extern int medium;
extern FILE *ps;


extern Widget toplevel, draww;
extern GC uniGC;

void SetColor(Pixmap image, Symbol * sym)
{

    if (strlen(sym->file) > 0) {
	if (!strcmp(sym->file, "-")) {
	    sym->brush = NULL;
	} else {
	    sym->brush = XCreateGC(XtDisplay(draww),
			   RootWindowOfScreen(XtScreen(draww)), 0, NULL);
	}
    } else {
	Display *dpy = XtDisplay(draww);
	Colormap cmap = DefaultColormapOfScreen(XtScreen(draww));
	char bufp[30];

	XColor color;

	color.red = (sym->r << 8);
	color.green = (sym->g << 8);
	color.blue = (sym->b << 8);
	if (!myXAllocColor(dpy, cmap, &color)) {
	    sprintf(bufp, "Can't alloc color");
	    XtWarning(bufp);
	    return;
	}
	sym->brush = XCreateGC(XtDisplay(draww),
			   RootWindowOfScreen(XtScreen(draww)), 0, NULL);

	if (sym->w > 1)
	    XSetLineAttributes(XtDisplay(draww),
		     sym->brush, sym->w, LineSolid, CapRound, JoinBevel);

	XSetForeground(dpy, sym->brush, color.pixel);
    }
}

ImageColorAllocate()
{
}

ImageFilledRectangle(Pixmap image, Position x1, Position y1, Position x2, Position y2, GC gc)
{
    if (gc)
	switch (medium) {
	case SCREEN:
	    XDrawRectangle(XtDisplay(toplevel), image, gc, x1, y1, x1 - x2, y1 - y2);
	    break;
	case COLORPS:
	    set_ps_color(gc);
	    fprintf(ps, "newpath		% Filled Rectangle\n%d %d moveto\n", x1, y1);
	    fprintf(ps, "%d %d lineto\n", x2, y1);
	    fprintf(ps, "%d %d lineto\n", x2, y2);
	    fprintf(ps, "%d %d lineto\n", x1, y2);
	    fprintf(ps, "closepath\neofill\n");
	    break;
	}
}

ImageFilledPolygon(Pixmap image, XPoint * points, int npoints, GC gc)
{
    int i;

    if (gc) {
	switch (medium) {
	case SCREEN:
	    XFillPolygon(XtDisplay(toplevel), image, gc, points, npoints,
			 Complex, CoordModeOrigin);
	    break;
	case COLORPS:
	    set_ps_color(gc);
	    fprintf(ps, "newpath		% Filled polygon\n%d %d moveto\n", points[0].x, points[0].y);
	    for (i = 1; i < npoints; i++)
		fprintf(ps, "%d %d lineto\n", points[i].x, points[i].y);
	    fprintf(ps, "closepath\neofill\n");
	    break;
	}
    }
}

ImagePolygon(Pixmap image, XPoint * points, int npoints, GC gc)
{
    int i;

    if (gc) {
	switch (medium) {
	case SCREEN:
	    XDrawLines(XtDisplay(toplevel), image, gc, points, npoints,
		       Complex /*, CoordModeOrigin */ );
	    break;
	case COLORPS:
	    set_ps_color(gc);
	    fprintf(ps, "newpath		% Polygon\n%d %d moveto\n", points[0].x, points[0].y);
	    for (i = 1; i < npoints; i++)
		fprintf(ps, "%d %d lineto\n", points[i].x, points[i].y);
	    fprintf(ps, "closepath\nstroke\n");
	    break;
	}
    }
}

ImageLine(Pixmap image, Position x1, Position y1, Position x2, Position y2, GC gc)
{
    if (gc)
	switch (medium) {
	case SCREEN:
	    XDrawLine(XtDisplay(toplevel), image, gc, x1, y1, x2, y2);
	    break;
	case COLORPS:
	    set_ps_color(gc);
	    if (x1 == x2 && y1 == y2) {
		fprintf(ps, "newpath\n%d %d 2 0 360 arc closepath fill\n", x1, y1);
		
	    } else {
		fprintf(ps, "newpath\n%d %d moveto\n%d %d lineto\nstroke\n\n",
			x1, y1, x2, y2);
	    }
	    break;
	}
}

ImageSetBrush()
{
}

ImageString(Pixmap image, XFontStruct * font, Position x, Position y, char *label, short color)
{
    switch (medium) {
    case SCREEN:
	set_font(uniGC, font);
	set_color(uniGC, "white");
	XDrawString(XtDisplay(draww), image, uniGC, x + 1, y + 1, label, strlen(label));
	set_color(uniGC, "black");
	XDrawString(XtDisplay(draww), image, uniGC, x, y, label, strlen(label));
	break;
    case COLORPS:
	fprintf(ps, "/Helvetica-Narrow findfont 11 scalefont setfont\n");
	fprintf(ps, "%d %d\n", y, x);
	fprintf(ps, "(%s) stringwidth\npop\n100 div sub exch moveto\n", label);
//    fprintf (ps, "(%s) stringwidth\npop\n2 div sub exch moveto\n", label);
	fprintf(ps, "gsave 1 -1 scale\n");
	fprintf(ps, "(%s) show\n", label);
	fprintf(ps, "grestore\n");
	break;
    }
}


ImageStringUp(Pixmap image, XFontStruct * font, Position x, Position y, char *label, short color)
{
//    fprintf(stderr, "label: %s\n", label);
}
